<?php
/**
 * Smarty plugin
 * @package Smarty
 * @subpackage plugins
 */

/**
 * Smarty {while} block function plugin
 *
 * Type:     block function<br>
 * Name:     while<br>
 * Date:     Mar 23 2006<br>
 * Purpose:  provide a while-loop<br>
 * Input:    var = string, name of a boolean variable
 * Examples: The following code will loop 1 time ;)
 *     <pre>
 *     {assign var="repeat" value=true}
 *     {while var="repeat"}
 *       hello world
 *       {assign var="repeat" value=false}
 *     {/while}
 *     </pre>
 * @author Misha Aizatulin <avatar@hot.ee>
 */

function smarty_block_while($params,  $content, &$smarty, &$repeat)
{
  if(!isset($params['var']))
    $smarty->trigger_error("while: missing parameter 'var'", E_USER_ERROR);

  // we have to take the name of the variable instead of
  // the variable itself, because parameters to blocks are
  // computed only once
  $repeat = $smarty->get_template_vars($params['var']);
 
  return $content;
}


/*
* Smarty plugin
* -------------------------------------------------------------
* File:     block.repeat.php
* Type:     block
* Name:     repeat
* Purpose:  repeat a template block a given number of times
* Parameters: count [required]  - number of times to repeat
*             assign [optional] - variable to collect output
* Author:   Scott Matthewman <scott@matthewman.net>
* -------------------------------------------------------------
*/
function smarty_block_repeat($params, $content, &$smarty)
{
    if (!empty($content)) {
        $intCount = intval($params['count']);
        if($intCount < 0) {
            $smarty->trigger_error("block: negative 'count' parameter");
            return;
        }
        
        $strRepeat = str_repeat($content, $intCount);
        if (!empty($params['assign'])) {
            $smarty->assign($params['assign'], $strRepeat);
        } else {
            echo $strRepeat;
        }
    }
}

function smarty_modifier_url($sString, $type = "") 
{
    global $WS, $LANG;
	
	$args = func_get_args();
	$index = 2;
	
	$sString = $LANG->url($sString);
	
	$url = $WS->url($sString, $type);
	
	for ($i=$index;$i<count($args);$i++)
	{
		$url .= $args[$i];
	}
	$url = mb_strtolower($url, "UTF-8");
	
	/*if (LANGUAGE_SHORT!="en") 
		$return = WEB_PATH.LANGUAGE_SHORT."/".$url;
	else*/
	$return = WEB_PATH.$url;

	return $return;	
} 

function smarty_block_dynamic($param, $content, &$smarty)
{
	return $content;
}

function smarty_function_translate($params, &$smarty) 
{
	global $LANG, $_LANG, $_MODULES, $cookie, $_MODULE;
	
	if (!isset($params['js'])) $params['js'] = 0;
	if (!isset($params['mod'])) $params['mod'] = false;
	$msg = false;

	$string = str_replace('\'', '\\\'', $params['s']);
	$key = $smarty->currentTemplate.'_'.md5($string);
	if ($params['mod'])
	{
		if (file_exists(TEMPLATE_LOCAL_ROOT.TEMPLATE.'/modules/'.$params['mod'].'/'.$cookie->id_lang.'.php'))
		{
			$translationsFile = TEMPLATE_LOCAL_ROOT.TEMPLATE.'/modules/'.$params['mod'].'/'.$cookie->id_lang.'.php';
			$modKey = '<{'.$params['mod'].'}'.TEMPLATE.'>'.$key;
		}
		else
		{
			$translationsFile = MODULES_DIR.$params['mod'].'/'.$cookie->id_lang.'.php';
			$modKey = '<{'.$params['mod'].'}default>'.$key;
		}
		
		if (@include_once($translationsFile))
			$_MODULES = array_merge($_MODULES, $_MODULE);

		$msg = (is_array($_MODULES) AND key_exists($modKey, $_MODULES)) ? html_entity_decode(($params['js'] ? addslashes($_MODULES[$modKey]) : stripslashes($_MODULES[$modKey])), ENT_COMPAT, 'UTF-8') : false;
	}
	if (!$msg)
		$msg = (is_array($_LANG) AND key_exists($key, $_LANG)) ? html_entity_decode(($params['js'] ? addslashes($_LANG[$key]) : stripslashes($_LANG[$key])), ENT_COMPAT, 'UTF-8') : $params['s'];

	if (isset($params['assign']) and $params['assign']!='') 
		$smarty->assign($params['assign'], $LANG->l($msg, $params));
	else
		return $LANG->l($msg, $params);
}

function get_main_cat($id)
{
	global $WS, $DB;
	$return = '';
	
	$sql = "SELECT a.*, b.* FROM `".DB_PREFIX."items_categories` a, `".DB_PREFIX."categories` b WHERE b.id=a.category_id AND a.item_id='".$id."' ";

	$sub_cat = $DB->get($sql);
	$parents = $WS->getParents($sub_cat['id']);
	$sql = "SELECT * FROM `".DB_PREFIX."categories` WHERE id='".$parents[0]."'";	
	$mainCat = $DB->get($sql);
	
	if ($mainCat['short_name']!="") $return = $mainCat['short_name'].'/';
	return $return;
}

function get_sub_cat($id)
{
	global $WS, $DB;
	$return = '';
	
	$sql = "SELECT a.*, b.* FROM `".DB_PREFIX."items_categories` a, `".DB_PREFIX."categories` b WHERE b.id=a.category_id AND a.item_id='".$id."' ";

	$sub_cat = $DB->get($sql);
	
	
	if ($sub_cat['short_name']!="") $return = $sub_cat['short_name'].'/';
	return $return;
}
function smarty_function_input($params, &$smarty) 
{
	global $_REQUEST, $DB;
	$theGrid = new thegrid($DB);
	return $theGrid->show_input($params, $_REQUEST[$params['name']]);
}


// register with smarty
$smarty->register_block('while', 'smarty_block_while');
$smarty->register_block('repeat', 'smarty_block_repeat');
$smarty->register_modifier('url', 'smarty_modifier_url');
$smarty->register_modifier('main_cat', 'get_main_cat');
$smarty->register_modifier('sub_cat', 'get_sub_cat');
$smarty->register_function('l', 'smarty_function_translate');
$smarty->register_function('input', 'smarty_function_input');
$smarty->register_block('dynamic', 'smarty_block_dynamic', false); 



?>