<?php

class Gsitemap extends Module
{
    var $_html = '';
    var $_postErrors = array();
	var $file_check = true;

    function Gsitemap()
    {
        $this->name = 'gsitemap';
        $this->tab = 'Tools';
        $this->version = 1.01;

        $this->_directory = dirname(__FILE__).'/../../';
        $this->_filename = $this->_directory.'sitemap.xml';
        $this->_filename_http = 'http://'.htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8').WEB_PATH.'sitemap.xml';

        parent::Module();

        /* The parent construct is required for translations */
		$this->page = basename(__FILE__, '.php');
        $this->displayName = $this->l('Google sitemap');
        $this->description = $this->l('Generate your Google sitemap file');
    }

    function uninstall()
    {
		global $DB;
        @unlink($this->_filename);
        parent::uninstall();
		
		// Hack for 'gsitemap' to uninstall it correctly
		$DB->query('
				DELETE FROM `'.DB_PREFIX.'module`
				WHERE `id_module` = '.intval($this->id));
		return true;
    }
	function install()
    {
        if (!parent::install() OR !$this->registerHook('content') or !$this->registerHook('userMenuLinks'))
			return false;
		return true;
    }
	
    function _postValidation()
    {
		@unlink($this->_filename);
		if (!$fp = @fopen($this->_filename, 'w'))
			$this->_postErrors[] = $this->l('Cannot create').' '.$this->_filename.' '.$this->l('file.');
		else
			fclose($fp);
    }
	function _crawl($page, $exclude = array())
	{
		$url_data = parse_url($page);
		
		$http = new Net_HTTP_Client();
		$http->Connect( $url_data['host'], 80 ) or die( "Connect problem" );
		$http->setProtocolVersion( "1.1" );
		$http->addHeader( "Host", $url_data['host'] );
		$http->addHeader( "Connection", "keep-alive" );
		
		$status = $http->Get( $url_data['path']."?".$url_data['query'] );
	
		$url_array = array();
	
		if( $status != 200 )
		{
			return false;
		}
		else
		{
			$urls = array();
			$str = $http->getBody();
			
			$reg_exp = "@href=\"([^\n\r\"]*)\"@i";
			$regex = preg_match_all($reg_exp,$str, $urls);
			
			foreach ($urls[1] as $url)
			{
				if (mb_strpos($url, $url_data['scheme']."://".$url_data['host'])===false and mb_strpos($url, "http://")===false and mb_strpos($url, "javascript:")===false)
				{
					$url = 'http://'.htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8').$url;
				}
				if (mb_strpos($url, "javascript:")===false and mb_strpos($url, $url_data['scheme']."://".$url_data['host'])!==false)
				{
					$testExclude = false;
					foreach ($exclude as $word)
					{
						if (mb_strpos($url, $word)!==false)
						{
							$testExclude = true;
						}
					}
					if (!$testExclude)
						$url_array[] = trim($url, "/");
				}
			}
			
			$url_array_processed = $url_array;
			if (!is_array($url_array_processed)) $url_array_processed = array();
			$url_array_processed = array_unique($url_array_processed);
			return $url_array_processed;
		}
	}
    function _postProcess()
    {
		global $DB;
		
		$homePageUrls = $this->_crawl('http://'.htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8').WEB_PATH, array("-wallpapers", "/templates/", "/modules/", "/view/", "/sifr/", "db.php"));

        $fp = fopen($this->_filename, 'w');
        $xmlDoc = new MiniXMLDoc();
		
		$xmlRoot =& $xmlDoc->getRoot();
		$xml =& $xmlRoot->createChild('urlset'); 
		$xml->attribute("xmlns", "http://www.google.com/schemas/sitemap/0.84");
		
		if (is_array($homePageUrls))
		{
			
			foreach ($homePageUrls as $homePageUrl)
			{
				$sitemap =& $xml->createChild('url');
	            $sitemap->createChild('loc', $homePageUrl);
	            $sitemap->createChild('priority', '0.9');
	            $sitemap->createChild('lastmod', date("Y-m-d"));
	            if (mb_strpos('http://'.htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8').WEB_PATH,$homePageUrl)!==false)
					$sitemap->createChild('changefreq', 'daily');
				else
					$sitemap->createChild('changefreq', 'daily');
			}
		}
		
		$categories = $DB->getAll('
		SELECT c.id, short_name, DATE_FORMAT(created, \'%Y-%m-%d\') AS modified
		FROM '.DB_PREFIX.'categories c
		WHERE c.enabled=1 and c.items>0 ');
		foreach($categories as $category)
        {
			$sitemap =& $xml->createChild('url');
            $sitemap->createChild('loc', 'http://'.htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8').htmlspecialchars(smarty_modifier_url($category['short_name'], 'cat')));
            $sitemap->createChild('priority', '0.9');
            $sitemap->createChild('lastmod', date("Y-m-d"));
            $sitemap->createChild('changefreq', 'weekly');
			unset($category);
        }
		unset($categories);
		
		$resolutions = $DB->getAll('
		SELECT *
		FROM '.DB_PREFIX.'resolutions WHERE 1 ');
        foreach($resolutions as $resolution)
        {
            $sitemap =& $xml->createChild('url');
            $sitemap->createChild('loc', 'http://'.htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8').htmlspecialchars(smarty_modifier_url($resolution['width']."x".$resolution['height'], 'resolution')));
            $sitemap->createChild('priority', '0.8');
            $sitemap->createChild('lastmod', date("Y-m-d"));
            $sitemap->createChild('changefreq', 'weekly');
        }
		unset($resolutions);

        /*$items = $DB->getAll('
		SELECT i.id, short_name, DATE_FORMAT(modified, \'%Y-%m-%d\') AS modified 
		FROM '.DB_PREFIX.'items i WHERE i.enabled=1 AND i.visible=1 ');
        foreach($items as $item)
        {
            $sitemap =& $xml->createChild('url');
            $sitemap->createChild('loc', 'http://'.htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8').htmlspecialchars(smarty_modifier_url($item['short_name'], 'item')));
            $sitemap->createChild('priority', '0.7');
            $sitemap->createChild('lastmod', $item['modified']);
            $sitemap->createChild('changefreq', 'weekly');
			
			
			#Maybe we can add the view resolutions for each wallpaper
			
			unset($item);
        }
		unset($items);*/
		

        $xmlString = $xmlDoc->toString();
        fwrite($fp, $xmlString, mb_strlen($xmlString));
        fclose($fp);

        $res = file_exists($this->_filename);
        $this->_html .= '<h3 class="'. ($res ? 'conf confirm' : 'alert error') .'" style="margin-bottom: 20px">';
        $this->_html .= $res ? $this->l('Sitemap file successfully generated') : $this->l('Error while creating sitemap file');
        $this->_html .= '</h3>';
		
		if ($res)
		{
			$http = new Net_HTTP_Client();
			$http->Connect( "www.google.com", 80 ) or die( "Connect problem" );
			$http->setProtocolVersion( "1.1" );
			$http->addHeader( "Host", "www.google.com" );
			$http->addHeader( "Connection", "keep-alive" );
			
			$status = $http->Get( "/webmasters/tools/ping?sitemap=".urlencode($this->_filename_http) );
			if( $status != 200 )
			{
				$this->_html .= '<h4>'.$this->l('Error submiting sitemap file to google').'</h4>';
			}
			else
			{
				$this->_html .= '<h4>'.$this->l('Sitemap file successfully sent to google').'</h4>';
			}
		}
    }
	
	/*function _postProcess()
    {
		global $DB;
		
		$homePageUrls = $this->_crawl('http://'.htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8').WEB_PATH, array("-wallpapers", "/templates/", "/modules/", "/view/", "/sifr/"));

        $fp = fopen($this->_filename, 'w');
        
		$xml = new SimpleXMLElement('<urlset xmlns="http://www.google.com/schemas/sitemap/0.84"></urlset>');
			
		foreach ($homePageUrls as $homePageUrl)
		{
			$sitemap = $xml->addChild('url');
            $sitemap->addChild('loc', $homePageUrl);
            $sitemap->addChild('priority', '0.9');
            $sitemap->addChild('lastmod', date("Y-m-d"));
            if (mb_strpos('http://'.htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8').WEB_PATH,$homePageUrl)!==false)
				$sitemap->addChild('changefreq', 'daily');
			else
				$sitemap->addChild('changefreq', 'daily');
		}
		
		$categories = $DB->getAll('
		SELECT c.id, short_name, DATE_FORMAT(created, \'%Y-%m-%d\') AS modified
		FROM '.DB_PREFIX.'categories c
		WHERE c.enabled=1 and c.items>0 ');
		foreach($categories as $category)
        {
			$sitemap = $xml->addChild('url');
            $sitemap->addChild('loc', 'http://'.htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8').htmlspecialchars(smarty_modifier_url($category['short_name'], 'cat')));
            $sitemap->addChild('priority', '0.9');
            $sitemap->addChild('lastmod', $category['modified']);
            $sitemap->addChild('changefreq', 'weekly');
			unset($category);
        }
		unset($categories);
		
		$resolutions = $DB->getAll('
		SELECT *
		FROM '.DB_PREFIX.'resolutions WHERE 1 ');
        foreach($resolutions as $resolution)
        {
            $sitemap = $xml->addChild('url');
            $sitemap->addChild('loc', 'http://'.htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8').htmlspecialchars(smarty_modifier_url($resolution['width']."x".$resolution['height'], 'resolution')));
            $sitemap->addChild('priority', '0.8');
            $sitemap->addChild('lastmod', date("Y-m-d"));
            $sitemap->addChild('changefreq', 'weekly');
        }
		unset($resolutions);

        $items = $DB->getAll('
		SELECT i.id, short_name, DATE_FORMAT(modified, \'%Y-%m-%d\') AS modified 
		FROM '.DB_PREFIX.'items i WHERE i.enabled=1 AND i.visible=1 ');
        foreach($items as $item)
        {
            $sitemap = $xml->addChild('url');
            $sitemap->addChild('loc', 'http://'.htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8').htmlspecialchars(smarty_modifier_url($item['short_name'], 'item')));
            $sitemap->addChild('priority', '0.7');
            $sitemap->addChild('lastmod', $item['modified']);
            $sitemap->addChild('changefreq', 'weekly');
			#Maybe we can add the view resolutions for each wallpaper
			
			unset($item);
        }
		unset($items);
		

        $xmlString = $xml->asXML();
        fwrite($fp, $xmlString, mb_strlen($xmlString));
        fclose($fp);

        $res = file_exists($this->_filename);
        $this->_html .= '<h3 class="'. ($res ? 'conf confirm' : 'alert error') .'" style="margin-bottom: 20px">';
        $this->_html .= $res ? $this->l('Sitemap file successfully generated') : $this->l('Error while creating sitemap file');
        $this->_html .= '</h3>';
		
		if ($res)
		{
			$http = new Net_HTTP_Client();
			$http->Connect( "www.google.com", 80 ) or die( "Connect problem" );
			$http->setProtocolVersion( "1.1" );
			$http->addHeader( "Host", "www.google.com" );
			$http->addHeader( "Connection", "keep-alive" );
			
			$status = $http->Get( "/webmasters/tools/ping?sitemap=".urlencode($this->_filename_http) );
			if( $status != 200 )
			{
				$this->_html .= '<h4>'.$this->l('Error submiting sitemap file to google').'</h4>';
			}
			else
			{
				$this->_html .= '<h4>'.$this->l('Sitemap file successfully sent to google').'</h4>';
			}
		}
    }*/
	
    function _displaySitemap()
    {
        if (file_exists($this->_filename))
        {			
            $fp = fopen($this->_filename, 'r');
            $fstat = fstat($fp);
            fclose($fp);
            $xml = simplexml_load_file($this->_filename);
            $nbPages = sizeof($xml->url);
			

            $this->_html .= '<p>'.$this->l('Your Google sitemap file is online at the following address:').'<br />
            <a href="'.$this->_filename_http.'"><b>'.$this->_filename_http.'</b></a></p><br />';

            //setlocale(LC_ALL, 'fr_FR@euro', 'fr_FR', 'fr');
            $this->_html .= $this->l('Update:').' <b>'.strftime('%A %d %B %Y %H:%M:%S').'</b><br />';
            $this->_html .= $this->l('Filesize:').' <b>'.number_format(($fstat['size']*.000001), 3).'mo</b><br />';
            $this->_html .= $this->l('Indexed pages:').' <b>'.$nbPages.'</b><br /><br />';
        }
    }

    function _displayForm()
    {
        $this->_html .=
        '<form action="'.$_SERVER['REQUEST_URI'].'" method="post">
			<input name="btnSubmit" ';
		if (!$this->file_check) $this->_html .=	' disabled="disabled" ';
		$this->_html .=	' class="button';
		if (!$this->file_check) $this->_html .=	' disabled';
		
		$this->_html .=	'" value="'.((!file_exists($this->_filename)) ? $this->l('Generate sitemap file') : $this->l('Update sitemap file')).'" type="submit" />
        </form>';
    }
    
    function getContent()
    {
        $this->_html .= '<h2>'.$this->l('Search Engine Optimization').'</h2>';

        $this->_html .= $this->l('See').' <a href="https://www.google.com/webmasters/tools/docs/en/about.html"> '.$this->l('this page').'</a> '.$this->l('for more information').'<br /><br />';

		if (!$this->file_check) $this->_html .= '<span class="warning">'.$this->l('Please create the empty sitemap file in the root of the website and give it write permissions.').'</span>';

        if (!empty($_POST))
        {
            $this->_postValidation();
            if (!sizeof($this->_postErrors))
                $this->_postProcess();
            else
                foreach ($this->_postErrors AS $err)
                    $this->_html .= '<div class="alert error">'.$err.'</div>';
        }
        else
            $this->_html .= '<br />';

        $this->_displaySitemap();
        $this->_displayForm();

        return $this->_html;
    }
	
	function hookContent($params)
	{
		global $step, $siteError;
		$output = '';
		if ($params['page_name']=='gsitemap')
		{
			
			if ($_SESSION["user_id"]=="" or $_SESSION["user_data"]["type"]!="admin")
			{
				redirect(smarty_modifier_url("login"));
			}
			if (file_exists($this->_filename) and is_writeable($this->_filename))
			{
				$this->file_check = true;
			}
			else
			{
				if (is_writeable($this->_directory) and !file_exists($this->_filename))
				{
					$this->file_check = true;
				}
				else
				{
					if (!file_exists($this->_filename)) $siteError->add($this->l('Sitemap file `%s` doesn\'t exist.', array('p1'=>$this->_filename)));
					if (!is_writeable($this->_filename)) $siteError->add($this->l('Can\'t write to sitemap file `%s`.', array('p1'=>$this->_filename)));
					$this->file_check = false;
				}
			}
			
			$output = $this->getContent();
		}
		return $output;
	}
	
	function hookUserMenuLinks($params)
	{
		global $smarty, $_SESSION;
		
		if ($_SESSION['user_data']['type']=="admin")
		{
			$output = '<li><a href="'.smarty_modifier_url("gsitemap").'">'.$this->l('Google Sitemap').'</a></li>';
		}
		return $output;
	}
	function _configure()
	{
		redirect(smarty_modifier_url("gsitemap"));
	}
	
}


?>